# chooseCourses.py
#
# Uses different CourseSelector subclasses to choose courses
# based on a given maximum effort.

from course import Course
from courseSelector import ValueGreedyCourseSelector
from courseSelector import EffortGreedyCourseSelector
from courseSelector import ValuePerEffortGreedyCourseSelector
from courseSelector import OptimalCourseSelector

def parseCoursesFromFile(filename):
    fin = open(filename, 'r')

    courses = []
    for line in fin:
        s = line.strip().split(',')
        course = Course(s[0], int(s[1]), int(s[2]))
        courses.append(course)

    fin.close()

    return courses

def chooseCourses(courses, maxEffort):
    valueGreedyCourseSelector = ValueGreedyCourseSelector(courses)
    effortGreedyCourseSelector = EffortGreedyCourseSelector(courses)
    valuePerEffortGreedyCourseSelector = ValuePerEffortGreedyCourseSelector(courses)

    print("Maximum effort:", maxEffort)

    print("\nValue greedy result:")
    res = valueGreedyCourseSelector.chooseCourses(maxEffort)
    print("  value:", res[0])
    print("  effort:", res[1])
    for course in res[2]:
        print("  " + str(course))

    print("\nEffort greedy result:")
    res = effortGreedyCourseSelector.chooseCourses(maxEffort)
    print("  value:", res[0])
    print("  effort:", res[1])
    for course in res[2]:
        print("  " + str(course))  

    print("\nValue-to-effort ratio greedy result:")
    res = valuePerEffortGreedyCourseSelector.chooseCourses(maxEffort)
    print("  value:", res[0])
    print("  effort:", res[1])
    for course in res[2]:
        print("  " + str(course))

if __name__ == "__main__":
    maxEffort = 14
    courses = parseCoursesFromFile("courseData.csv")
    chooseCourses(courses, maxEffort)
